module win32.directx.directdraw;

import std.c.windows.windows;
import std.c.windows.com;
import win32.wingdi;
import win32.oleextra;

// direct draw basic interfaces etc (not fully implemented)

/*
 * some of these structures might need align-ing with align(1)
 */

extern(Windows) {

/*! a few types you might need */
//struct IDirectDraw;
//struct IDirectDrawSurface;
//struct IDirectDrawPalette;
//struct IDirectDrawClipper;

enum { DD_OK = 0 }

/* N.B. IDirectDraw is a pointer to the com struct */

alias IDirectDraw             LPDIRECTDRAW;
//alias IDirectDraw2            LPDIRECTDRAW2;
alias IDirectDrawSurface      LPDIRECTDRAWSURFACE;
//alias IDirectDrawSurface2     LPDIRECTDRAWSURFACE2;
//alias IDirectDrawSurface3     LPDIRECTDRAWSURFACE3;

alias IDirectDrawPalette      LPDIRECTDRAWPALETTE;
alias IDirectDrawClipper      LPDIRECTDRAWCLIPPER;
alias IDirectDrawColorControl LPDIRECTDRAWCOLORCONTROL;

//typedef _DDFXROP         * LPDDFXROP;

struct DDPIXELFORMAT
{
    DWORD	dwSize;			// size of structure
    DWORD	dwFlags;		// pixel format flags
    DWORD	dwFourCC;		// (FOURCC code)
	 /*
    union
    {
	DWORD	dwRGBBitCount;		// how many bits per pixel
	DWORD	dwYUVBitCount;		// how many bits per pixel
	DWORD	dwZBufferBitDepth;	// how many bits for z buffers
	DWORD	dwAlphaBitDepth;	// how many bits for alpha channels
    };
	 */
	DWORD	dwRGBBitCount;		// how many bits per pixel
	/*
    union
    {
	DWORD	dwRBitMask;		// mask for red bit
	DWORD	dwYBitMask;		// mask for Y bits
    };
	 */
	DWORD	dwRBitMask;		// mask for red bit
	/*
    union
    {
	DWORD	dwGBitMask;		// mask for green bits
	DWORD	dwUBitMask;		// mask for U bits
    };
	 */
	DWORD	dwGBitMask;		// mask for green bits
	/*
    union
    {
	DWORD	dwBBitMask;		// mask for blue bits
	DWORD	dwVBitMask;		// mask for V bits
    };
	 */
	DWORD	dwBBitMask;		// mask for blue bits
	/*
    union
    {
	DWORD	dwRGBAlphaBitMask;	// mask for alpha channel
	DWORD	dwYUVAlphaBitMask;	// mask for alpha channel
	DWORD	dwRGBZBitMask;		// mask for Z channel
	DWORD	dwYUVZBitMask;		// mask for Z channel
    };
	 */
	DWORD	dwRGBAlphaBitMask;	// mask for alpha channel
}
alias DDPIXELFORMAT * LPDDPIXELFORMAT;

struct DDCOLORCONTROL
{
    DWORD 		dwSize;
    DWORD		dwFlags;
    LONG		lBrightness;
    LONG		lContrast;
    LONG		lHue;
    LONG 		lSaturation;
    LONG		lSharpness;
    LONG		lGamma;
    LONG		lColorEnable;
    DWORD		dwReserved1;
}
alias DDCOLORCONTROL  * LPDDCOLORCONTROL;

struct DDCOLORKEY
{
    DWORD	dwColorSpaceLowValue;	// low boundary of color space that is to
					// be treated as Color Key, inclusive
    DWORD	dwColorSpaceHighValue;	// high boundary of color space that is
					// to be treated as Color Key, inclusive
};
alias DDCOLORKEY * LPDDCOLORKEY;


struct DDSCAPS
{
	DWORD dwCaps;	// capabilities of surface wanted
};
alias DDSCAPS * LPDDSCAPS;


const int DD_ROP_SPACE = (256/32); // space required to store ROP array

struct DDCAPS
{
/*  0*/ DWORD	dwSize;			// size of the DDDRIVERCAPS structure
/*  4*/ DWORD	dwCaps;			// driver specific capabilities
/*  8*/ DWORD	dwCaps2;		// more driver specific capabilites
/*  c*/ DWORD	dwCKeyCaps;		// color key capabilities of the surface
/* 10*/ DWORD	dwFXCaps;		// driver specific stretching and effects capabilites
/* 14*/ DWORD	dwFXAlphaCaps;		// alpha driver specific capabilities
/* 18*/ DWORD	dwPalCaps;		// palette capabilities
/* 1c*/ DWORD	dwSVCaps;		// stereo vision capabilities
/* 20*/ DWORD	dwAlphaBltConstBitDepths;	// DDBD_2,4,8
/* 24*/ DWORD	dwAlphaBltPixelBitDepths;	// DDBD_1,2,4,8
/* 28*/ DWORD	dwAlphaBltSurfaceBitDepths;	// DDBD_1,2,4,8
/* 2c*/ DWORD	dwAlphaOverlayConstBitDepths;	// DDBD_2,4,8
/* 30*/ DWORD	dwAlphaOverlayPixelBitDepths;	// DDBD_1,2,4,8
/* 34*/ DWORD	dwAlphaOverlaySurfaceBitDepths; // DDBD_1,2,4,8
/* 38*/ DWORD	dwZBufferBitDepths;		// DDBD_8,16,24,32
/* 3c*/ DWORD	dwVidMemTotal;		// total amount of video memory
/* 40*/ DWORD	dwVidMemFree;		// amount of free video memory
/* 44*/ DWORD	dwMaxVisibleOverlays;	// maximum number of visible overlays
/* 48*/ DWORD	dwCurrVisibleOverlays;	// current number of visible overlays
/* 4c*/ DWORD	dwNumFourCCCodes;	// number of four cc codes
/* 50*/ DWORD	dwAlignBoundarySrc;	// source rectangle alignment
/* 54*/ DWORD	dwAlignSizeSrc;		// source rectangle byte size
/* 58*/ DWORD	dwAlignBoundaryDest;	// dest rectangle alignment
/* 5c*/ DWORD	dwAlignSizeDest;	// dest rectangle byte size
/* 60*/ DWORD	dwAlignStrideAlign;	// stride alignment
/* 64*/ DWORD	dwRops[DD_ROP_SPACE];	// ROPS supported
/* 84*/ DDSCAPS	ddsCaps;		// DDSCAPS structure has all the general capabilities
/* 88*/ DWORD	dwMinOverlayStretch;	// minimum overlay stretch factor multiplied by 1000, eg 1000 == 1.0, 1300 == 1.3
/* 8c*/ DWORD	dwMaxOverlayStretch;	// maximum overlay stretch factor multiplied by 1000, eg 1000 == 1.0, 1300 == 1.3
/* 90*/ DWORD	dwMinLiveVideoStretch;	// minimum live video stretch factor multiplied by 1000, eg 1000 == 1.0, 1300 == 1.3
/* 94*/ DWORD	dwMaxLiveVideoStretch;	// maximum live video stretch factor multiplied by 1000, eg 1000 == 1.0, 1300 == 1.3
/* 98*/ DWORD	dwMinHwCodecStretch;	// minimum hardware codec stretch factor multiplied by 1000, eg 1000 == 1.0, 1300 == 1.3
/* 9c*/ DWORD	dwMaxHwCodecStretch;	// maximum hardware codec stretch factor multiplied by 1000, eg 1000 == 1.0, 1300 == 1.3
/* a0*/ DWORD	dwReserved1;		// reserved
/* a4*/ DWORD	dwReserved2;		// reserved
/* a8*/ DWORD	dwReserved3;		// reserved
/* ac*/ DWORD	dwSVBCaps;		// driver specific capabilities for System->Vmem blts
/* b0*/ DWORD	dwSVBCKeyCaps;		// driver color key capabilities for System->Vmem blts
/* b4*/ DWORD	dwSVBFXCaps;		// driver FX capabilities for System->Vmem blts
/* b8*/ DWORD	dwSVBRops[DD_ROP_SPACE];// ROPS supported for System->Vmem blts
/* d8*/ DWORD	dwVSBCaps;		// driver specific capabilities for Vmem->System blts
/* dc*/ DWORD	dwVSBCKeyCaps;		// driver color key capabilities for Vmem->System blts
/* e0*/ DWORD	dwVSBFXCaps;		// driver FX capabilities for Vmem->System blts
/* e4*/ DWORD	dwVSBRops[DD_ROP_SPACE];// ROPS supported for Vmem->System blts
/*104*/ DWORD	dwSSBCaps;		// driver specific capabilities for System->System blts
/*108*/ DWORD	dwSSBCKeyCaps;		// driver color key capabilities for System->System blts
/*10c*/ DWORD	dwSSBFXCaps;		// driver FX capabilities for System->System blts
/*110*/ DWORD	dwSSBRops[DD_ROP_SPACE];// ROPS supported for System->System blts
//#if       DIRECTDRAW_VERSION >= 0x0500
///*130*/ DWORD	dwMaxVideoPorts;	// maximum number of usable video ports
///*134*/ DWORD	dwCurrVideoPorts;	// current number of video ports used
///*138*/ DWORD	dwSVBCaps2;		// more driver specific capabilities for System->Vmem blts
///*13c*/ DWORD	dwNLVBCaps;		  // driver specific capabilities for non-local->local vidmem blts
///*140*/ DWORD	dwNLVBCaps2;		  // more driver specific capabilities non-local->local vidmem blts
///*144*/ DWORD	dwNLVBCKeyCaps;		  // driver color key capabilities for non-local->local vidmem blts
///*148*/ DWORD	dwNLVBFXCaps;		  // driver FX capabilities for non-local->local blts
///*14c*/ DWORD	dwNLVBRops[DD_ROP_SPACE]; // ROPS supported for non-local->local blts
//#else  /* DIRECTDRAW_VERSION >= 0x0500 */
/*130*/ DWORD	dwReserved4;		// reserved
/*134*/ DWORD	dwReserved5;		// reserved
/*138*/ DWORD	dwReserved6;		// reserved
//#endif /* DIRECTDRAW_VERSION >= 0x0500 */
}

struct DDBLTFX
{
    DWORD	dwSize;				// size of structure
    DWORD	dwDDFX;				// FX operations
    DWORD	dwROP;				// Win32 raster operations
    DWORD	dwDDROP;			// Raster operations new for DirectDraw
    DWORD	dwRotationAngle;		// Rotation angle for blt
    DWORD	dwZBufferOpCode;		// ZBuffer compares
    DWORD	dwZBufferLow;			// Low limit of Z buffer
    DWORD	dwZBufferHigh;			// High limit of Z buffer
    DWORD	dwZBufferBaseDest;		// Destination base value
    DWORD	dwZDestConstBitDepth;		// Bit depth used to specify Z constant for destination
	 /*
    union
    {
	DWORD	dwZDestConst;			// Constant to use as Z buffer for dest
	LPDIRECTDRAWSURFACE lpDDSZBufferDest;	// Surface to use as Z buffer for dest
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSZBufferDest;	// Surface to use as Z buffer for dest
    DWORD	dwZSrcConstBitDepth;		// Bit depth used to specify Z constant for source
	 /*
    union
    {
	DWORD	dwZSrcConst;			// Constant to use as Z buffer for src
	LPDIRECTDRAWSURFACE lpDDSZBufferSrc;	// Surface to use as Z buffer for src
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSZBufferSrc;	// Surface to use as Z buffer for src
    DWORD	dwAlphaEdgeBlendBitDepth;	// Bit depth used to specify constant for alpha edge blend
    DWORD	dwAlphaEdgeBlend;		// Alpha for edge blending
    DWORD	dwReserved;
    DWORD	dwAlphaDestConstBitDepth;	// Bit depth used to specify alpha constant for destination
	 /*
    union
    {
	DWORD	dwAlphaDestConst;		// Constant to use as Alpha Channel
	LPDIRECTDRAWSURFACE lpDDSAlphaDest;	// Surface to use as Alpha Channel
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSAlphaDest;	// Surface to use as Alpha Channel
    DWORD	dwAlphaSrcConstBitDepth;	// Bit depth used to specify alpha constant for source
	 /*
    union
    {
	DWORD	dwAlphaSrcConst;		// Constant to use as Alpha Channel
	LPDIRECTDRAWSURFACE lpDDSAlphaSrc;	// Surface to use as Alpha Channel
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSAlphaSrc;	// Surface to use as Alpha Channel
	/*
    union
    {
	DWORD	dwFillColor;			// color in RGB or Palettized
	DWORD   dwFillDepth;                    // depth value for z-buffer
	DWORD	dwFillPixel;			// pixel value for RGBA or RGBZ
	LPDIRECTDRAWSURFACE lpDDSPattern;	// Surface to use as pattern
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSPattern;	// Surface to use as pattern
    DDCOLORKEY	ddckDestColorkey;		// DestColorkey override
    DDCOLORKEY	ddckSrcColorkey;		// SrcColorkey override
}

alias DDBLTFX * LPDDBLTFX;

alias DDCAPS * LPDDCAPS;

struct DDSURFACEDESC
{
	DWORD		dwSize;			// size of the DDSURFACEDESC structure
	DWORD		dwFlags;		// determines what fields are valid
	DWORD		dwHeight;		// height of surface to be created
	DWORD		dwWidth;		// width of input surface

	/*
	union
	{
		LONG		lPitch;			// distance to start of next line (return value only)
		DWORD           dwLinearSize;           // Formless late-allocated optimized surface size
	};
	*/
	LONG		lPitch;			// distance to start of next line (return value only)

	DWORD		dwBackBufferCount;	// number of back buffers requested

/*	union
	{
		DWORD		dwMipMapCount;          // number of mip-map levels requested
		DWORD		dwZBufferBitDepth;	// depth of Z buffer requested
		DWORD		dwRefreshRate;		// refresh rate (used when display mode is described)
	};
	*/
	DWORD		dwRefreshRate;		// refresh rate (used when display mode is described)

	DWORD		dwAlphaBitDepth;	// depth of alpha buffer requested
	DWORD		dwReserved;		// reserved
	LPVOID		lpSurface;		// pointer to the associated surface memory
	DDCOLORKEY		ddckCKDestOverlay;	// color key for destination overlay use
	DDCOLORKEY		ddckCKDestBlt;		// color key for destination blt use
	DDCOLORKEY		ddckCKSrcOverlay;	// color key for source overlay use
	DDCOLORKEY		ddckCKSrcBlt;		// color key for source blt use
	DDPIXELFORMAT	ddpfPixelFormat;	// pixel format description of the surface
	DDSCAPS		ddsCaps;		// direct draw surface capabilities
}
alias DDSURFACEDESC * LPDDSURFACEDESC;

struct DDOVERLAYFX
{
    DWORD	dwSize;				// size of structure
    DWORD	dwAlphaEdgeBlendBitDepth;	// Bit depth used to specify constant for alpha edge blend
    DWORD	dwAlphaEdgeBlend;		// Constant to use as alpha for edge blend
    DWORD	dwReserved;
    DWORD	dwAlphaDestConstBitDepth;	// Bit depth used to specify alpha constant for destination
	 /*
    union
    {
	DWORD	dwAlphaDestConst;		// Constant to use as alpha channel for dest
	LPDIRECTDRAWSURFACE lpDDSAlphaDest;	// Surface to use as alpha channel for dest
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSAlphaDest;	// Surface to use as alpha channel for dest

    DWORD	dwAlphaSrcConstBitDepth;	// Bit depth used to specify alpha constant for source
	 /*
    union
    {
	DWORD	dwAlphaSrcConst;		// Constant to use as alpha channel for src
	LPDIRECTDRAWSURFACE lpDDSAlphaSrc;	// Surface to use as alpha channel for src
    };
	 */
	LPDIRECTDRAWSURFACE lpDDSAlphaSrc;	// Surface to use as alpha channel for src

    DDCOLORKEY	dckDestColorkey;		// DestColorkey override
    DDCOLORKEY	dckSrcColorkey;			// DestColorkey override
    DWORD       dwDDFX;                         // Overlay FX
    DWORD	dwFlags;			// flags
}

alias DDOVERLAYFX * LPDDOVERLAYFX;


struct DDBLTBATCH
{
    LPRECT		lprDest;
    LPDIRECTDRAWSURFACE	lpDDSSrc;
    LPRECT		lprSrc;
    DWORD		dwFlags;
    LPDDBLTFX		lpDDBltFx;
}

alias DDBLTBATCH * LPDDBLTBATCH;


/* !!! These should work they are (PASCAL * LP...)() which is __stdcall on win32 */
alias HRESULT (* LPDDENUMMODESCALLBACK)(LPDDSURFACEDESC, LPVOID);
alias HRESULT (* LPDDENUMSURFACESCALLBACK)(LPDIRECTDRAWSURFACE, LPDDSURFACEDESC, LPVOID);


interface IDirectDraw : IUnknown
{
	/*** IUnknown methods ***/
//	HRESULT QueryInterface( REFIID riid, LPVOID * ppvObj);
//	ULONG   AddRef();
//	ULONG   Release();
	 /*** IDirectDraw methods ***/
	HRESULT Compact();
	HRESULT CreateClipper( DWORD, LPDIRECTDRAWCLIPPER *, IUnknown * );
	HRESULT CreatePalette( DWORD, LPPALETTEENTRY, LPDIRECTDRAWPALETTE *, IUnknown * );
	HRESULT CreateSurface( LPDDSURFACEDESC, LPDIRECTDRAWSURFACE *, IUnknown *);
	HRESULT DuplicateSurface( LPDIRECTDRAWSURFACE, LPDIRECTDRAWSURFACE * );
	HRESULT EnumDisplayModes( DWORD, LPDDSURFACEDESC, LPVOID, LPDDENUMMODESCALLBACK );
	HRESULT EnumSurfaces( DWORD, LPDDSURFACEDESC, LPVOID,LPDDENUMSURFACESCALLBACK );
	HRESULT FlipToGDISurface();
	HRESULT GetCaps( LPDDCAPS, LPDDCAPS );
	HRESULT GetDisplayMode( LPDDSURFACEDESC );
	HRESULT GetFourCCCodes( LPDWORD, LPDWORD );
	HRESULT GetGDISurface( LPDIRECTDRAWSURFACE * );
	HRESULT GetMonitorFrequency( LPDWORD );
	HRESULT GetScanLine( LPDWORD );
	HRESULT GetVerticalBlankStatus( LPBOOL );
	HRESULT Initialize( GUID * );
	HRESULT RestoreDisplayMode();
	HRESULT SetCooperativeLevel( HWND, DWORD );
	HRESULT SetDisplayMode( DWORD, DWORD,DWORD );
	HRESULT WaitForVerticalBlank( DWORD, HANDLE );
}

interface IDirectDrawPalette : IUnknown
{
	/*** IUnknown methods ***/
//	HRESULT QueryInterface( REFIID riid, LPVOID * ppvObj);
//	ULONG   AddRef();
//	ULONG   Release();
	/*** IDirectDrawPalette methods ***/
	HRESULT GetCaps( LPDWORD );
	HRESULT GetEntries( DWORD,DWORD,DWORD,LPPALETTEENTRY );
	HRESULT Initialize( LPDIRECTDRAW, DWORD, LPPALETTEENTRY );
	HRESULT SetEntries( DWORD,DWORD,DWORD,LPPALETTEENTRY );
}

interface IDirectDrawClipper : IUnknown
{
	/*** IUnknown methods ***/
//	HRESULT QueryInterface( REFIID riid, LPVOID * ppvObj);
//	ULONG   AddRef();
//	ULONG   Release();
	/*** IDirectDrawClipper methods ***/
	HRESULT GetClipList( LPRECT, LPRGNDATA, LPDWORD );
	HRESULT GetHWnd( HWND * );
	HRESULT Initialize( LPDIRECTDRAW, DWORD );
	HRESULT IsClipListChanged( BOOL * );
	HRESULT SetClipList( LPRGNDATA, DWORD );
	HRESULT SetHWnd( DWORD, HWND );
}

interface IDirectDrawSurface : IUnknown
{
	/*** IUnknown methods ***/
//	HRESULT QueryInterface( REFIID riid, LPVOID * ppvObj);
//	ULONG   AddRef();
//	ULONG   Release();
	/*** IDirectDrawSurface methods ***/
	HRESULT AddAttachedSurface( LPDIRECTDRAWSURFACE );
	HRESULT AddOverlayDirtyRect( LPRECT );
	HRESULT Blt( LPRECT,LPDIRECTDRAWSURFACE, LPRECT,DWORD, LPDDBLTFX );
	HRESULT BltBatch( LPDDBLTBATCH, DWORD, DWORD );
	HRESULT BltFast( DWORD,DWORD,LPDIRECTDRAWSURFACE, LPRECT,DWORD );
	HRESULT DeleteAttachedSurface( DWORD,LPDIRECTDRAWSURFACE );
	HRESULT EnumAttachedSurfaces( LPVOID,LPDDENUMSURFACESCALLBACK );
	HRESULT EnumOverlayZOrders( DWORD,LPVOID,LPDDENUMSURFACESCALLBACK );
	HRESULT Flip( LPDIRECTDRAWSURFACE, DWORD );
	HRESULT GetAttachedSurface( LPDDSCAPS, LPDIRECTDRAWSURFACE * );
	HRESULT GetBltStatus( DWORD );
	HRESULT GetCaps( LPDDSCAPS );
	HRESULT GetClipper( LPDIRECTDRAWCLIPPER * );
	HRESULT GetColorKey( DWORD, LPDDCOLORKEY );
	HRESULT GetDC( HDC * );
	HRESULT GetFlipStatus( DWORD );
	HRESULT GetOverlayPosition( LPLONG, LPLONG );
	HRESULT GetPalette( LPDIRECTDRAWPALETTE * );
	HRESULT GetPixelFormat( LPDDPIXELFORMAT );
	HRESULT GetSurfaceDesc( LPDDSURFACEDESC );
	HRESULT Initialize( LPDIRECTDRAW, LPDDSURFACEDESC);
	HRESULT IsLost();
	HRESULT Lock(LPRECT,LPDDSURFACEDESC,DWORD,HANDLE );
	HRESULT ReleaseDC( HDC );
	HRESULT Restore();
	HRESULT SetClipper( LPDIRECTDRAWCLIPPER );
	HRESULT SetColorKey( DWORD, LPDDCOLORKEY );
	HRESULT SetOverlayPosition( LONG, LONG );
	HRESULT SetPalette( LPDIRECTDRAWPALETTE );
	HRESULT Unlock( LPVOID );
	HRESULT UpdateOverlay( LPRECT, LPDIRECTDRAWSURFACE,LPRECT,DWORD, LPDDOVERLAYFX );
	HRESULT UpdateOverlayDisplay( DWORD );
	HRESULT UpdateOverlayZOrder( DWORD, LPDIRECTDRAWSURFACE );
};

interface IDirectDrawColorControl : IUnknown
{
	/*** IUnknown methods ***/
//	HRESULT QueryInterface( REFIID riid, LPVOID * ppvObj);
//	ULONG   AddRef();
//	ULONG   Release();
	/*** IDirectDrawColorControl methods ***/
	HRESULT GetColorControls( LPDDCOLORCONTROL );
	HRESULT SetColorControls( LPDDCOLORCONTROL );
};

} // extern (Windows)

/* ddraw.dll create function */
extern(Windows) typedef HRESULT (* ddCreateFunc)(GUID *, LPDIRECTDRAW *, IUnknown *);

enum : DWORD {
/* some of the defines (enough for basic testing) */
/*
 * Exclusive mode owner will be responsible for the entire primary surface.
 * GDI can be ignored. used with DD
 */
DDSCL_FULLSCREEN                       = 0x00000001l,

/*
 * allow CTRL_ALT_DEL to work while in fullscreen exclusive mode
 */
DDSCL_ALLOWREBOOT                      = 0x00000002l,

/*
 * prevents DDRAW from modifying the application window.
 * prevents DDRAW from minimize/restore the application window on activation.
 */
DDSCL_NOWINDOWCHANGES                   = 0x00000004l,

/*
 * app wants to work as a regular Windows application
 */
DDSCL_NORMAL                            = 0x00000008l,

/*
 * app wants exclusive access
 */
DDSCL_EXCLUSIVE                         = 0x00000010l,


/*
 * app can deal with non-windows display modes
 */
DDSCL_ALLOWMODEX                        = 0x00000040l,

/*
 * this window will receive the focus messages
 */
DDSCL_SETFOCUSWINDOW                    = 0x00000080l,

/*
 * this window is associated with the DDRAW object and will
 * cover the screen in fullscreen mode
 */
DDSCL_SETDEVICEWINDOW                   = 0x00000100l,

/*
 * app wants DDRAW to create a window to be associated with the
 * DDRAW object
 */
DDSCL_CREATEDEVICEWINDOW                = 0x00000200l

}


/*============================================================================
 *
 * Direct Draw Capability Flags
 *
 * These flags are used to describe the capabilities of a given Surface.
 * All flags are bit flags.
 *
 *==========================================================================*/

/****************************************************************************
 *
 * DIRECTDRAWSURFACE CAPABILITY FLAGS
 *
 ****************************************************************************/
enum :uint {
/*
 * This bit is reserved. It should not be specified.
 */
DDSCAPS_RESERVED1			 = 0x00000001,

/*
 * Indicates that this surface contains alpha-only information.
 * (To determine if a surface is RGBA/YUVA, the pixel format must be
 * interrogated.)
 */
DDSCAPS_ALPHA				= 0x00000002l,

/*
 * Indicates that this surface is a backbuffer.	 It is generally
 * set by CreateSurface when the DDSCAPS_FLIP capability bit is set.
 * It indicates that this surface is THE back buffer of a surface
 * flipping structure.	DirectDraw supports N surfaces in a
 * surface flipping structure.	Only the surface that immediately
 * precedeces the DDSCAPS_FRONTBUFFER has this capability bit set.
 * The other surfaces are identified as back buffers by the presence
 * of the DDSCAPS_FLIP capability, their attachment order, and the
 * absence of the DDSCAPS_FRONTBUFFER and DDSCAPS_BACKBUFFER
 * capabilities.  The bit is sent to CreateSurface when a standalone
 * back buffer is being created.  This surface could be attached to
 * a front buffer and/or back buffers to form a flipping surface
 * structure after the CreateSurface call.  See AddAttachments for
 * a detailed description of the behaviors in this case.
 */
DDSCAPS_BACKBUFFER			= 0x00000004,

/*
 * Indicates a complex surface structure is being described.  A
 * complex surface structure results in the creation of more than
 * one surface.	 The additional surfaces are attached to the root
 * surface.  The complex structure can only be destroyed by
 * destroying the root.
 */
DDSCAPS_COMPLEX				 = 0x00000008,

/*
 * Indicates that this surface is a part of a surface flipping structure.
 * When it is passed to CreateSurface the DDSCAPS_FRONTBUFFER and
 * DDSCAP_BACKBUFFER bits are not set.	They are set by CreateSurface
 * on the resulting creations.	The dwBackBufferCount field in the
 * DDSURFACEDESC structure must be set to at least 1 in order for
 * the CreateSurface call to succeed.  The DDSCAPS_COMPLEX capability
 * must always be set with creating multiple surfaces through CreateSurface.
 */
DDSCAPS_FLIP				 = 0x00000010,

/*
 * Indicates that this surface is THE front buffer of a surface flipping
 * structure.  It is generally set by CreateSurface when the DDSCAPS_FLIP
 * capability bit is set.
 * If this capability is sent to CreateSurface then a standalonw front buffer
 * is created.	This surface will not have the DDSCAPS_FLIP capability.
 * It can be attached to other back buffers to form a flipping structure.
 * See AddAttachments for a detailed description of the behaviors in this
 * case.
 */
DDSCAPS_FRONTBUFFER			= 0x00000020,

/*
 * Indicates that this surface is any offscreen surface that is not an overlay,
 * texture, zbuffer, front buffer, back buffer, or alpha surface.  It is used
 * to identify plain vanilla surfaces.
 */
DDSCAPS_OFFSCREENPLAIN		= 0x00000040,

/*
 * Indicates that this surface is an overlay.  It may or may not be directly visible
 * depending on whether or not it is currently being overlayed onto the primary
 * surface.  DDSCAPS_VISIBLE can be used to determine whether or not it is being
 * overlayed at the moment.
 */
DDSCAPS_OVERLAY				= 0x00000080,

/*
 * Indicates that unique DirectDrawPalette objects can be created and
 * attached to this surface.
 */
DDSCAPS_PALETTE				= 0x00000100,

/*
 * Indicates that this surface is the primary surface.	The primary
 * surface represents what the user is seeing at the moment.
 */
DDSCAPS_PRIMARYSURFACE		= 0x00000200,

/*
 * Indicates that this surface is the primary surface for the left eye.
 * The primary surface for the left eye represents what the user is seeing
 * at the moment with the users left eye.  When this surface is created the
 * DDSCAPS_PRIMARYSURFACE represents what the user is seeing with the users
 * right eye.
 */
DDSCAPS_PRIMARYSURFACELEFT		= 0x00000400,

/*
 * Indicates that this surface memory was allocated in system memory
 */
DDSCAPS_SYSTEMMEMORY			 = 0x00000800,

/*
 * Indicates that this surface can be used as a 3D texture.  It does not
 * indicate whether or not the surface is being used for that purpose.
 */
DDSCAPS_TEXTURE				= 0x00001000,

/*
 * Indicates that a surface may be a destination for 3D rendering.  This
 * bit must be set in order to query for a Direct3D Device Interface
 * from this surface.
 */
DDSCAPS_3DDEVICE				= 0x00002000,

/*
 * Indicates that this surface exists in video memory.
 */
DDSCAPS_VIDEOMEMORY			= 0x00004000,

/*
 * Indicates that changes made to this surface are immediately visible.
 * It is always set for the primary surface and is set for overlays while
 * they are being overlayed and texture maps while they are being textured.
 */
DDSCAPS_VISIBLE				= 0x00008000,

/*
 * Indicates that only writes are permitted to the surface.  Read accesses
 * from the surface may or may not generate a protection fault, but the
 * results of a read from this surface will not be meaningful.	READ ONLY.
 */
DDSCAPS_WRITEONLY			 = 0x00010000,

/*
 * Indicates that this surface is a z buffer. A z buffer does not contain
 * displayable information.  Instead it contains bit depth information that is
 * used to determine which pixels are visible and which are obscured.
 */
DDSCAPS_ZBUFFER				= 0x00020000,

/*
 * Indicates surface will have a DC associated long term
 */
DDSCAPS_OWNDC				 = 0x00040000,

/*
 * Indicates surface should be able to receive live video
 */
DDSCAPS_LIVEVIDEO			 = 0x00080000,

/*
 * Indicates surface should be able to have a stream decompressed
 * to it by the hardware.
 */
DDSCAPS_HWCODEC				= 0x00100000,

/*
 * Surface is a ModeX surface.
 *
 */
DDSCAPS_MODEX				 = 0x00200000,

/*
 * Indicates surface is one level of a mip-map. This surface will
 * be attached to other DDSCAPS_MIPMAP surfaces to form the mip-map.
 * This can be done explicitly, by creating a number of surfaces and
 * attaching them with AddAttachedSurface or by implicitly by CreateSurface.
 * If this bit is set then DDSCAPS_TEXTURE must also be set.
 */
DDSCAPS_MIPMAP				= 0x00400000,

/*
 * This bit is reserved. It should not be specified.
 */
DDSCAPS_RESERVED2			= 0x00800000,


/*
 * Indicates that memory for the surface is not allocated until the surface
 * is loaded (via the Direct3D texture Load() function).
 */
DDSCAPS_ALLOCONLOAD		= 0x04000000,

/*
 * Indicates that the surface will recieve data from a video port.
 */
DDSCAPS_VIDEOPORT			= 0x08000000,

/*
 * Indicates that a video memory surface is resident in true, local video
 * memory rather than non-local video memory. If this flag is specified then
 * so must DDSCAPS_VIDEOMEMORY. This flag is mutually exclusive with
 * DDSCAPS_NONLOCALVIDMEM.
 */
DDSCAPS_LOCALVIDMEM		= 0x10000000,

/*
 * Indicates that a video memory surface is resident in non-local video
 * memory rather than true, local video memory. If this flag is specified
 * then so must DDSCAPS_VIDEOMEMORY. This flag is mutually exclusive with
 * DDSCAPS_LOCALVIDMEM.
 */
DDSCAPS_NONLOCALVIDMEM	= 0x20000000,

/*
 * Indicates that this surface is a standard VGA mode surface, and not a
 * ModeX surface. (This flag will never be set in combination with the
 * DDSCAPS_MODEX flag).
 */
DDSCAPS_STANDARDVGAMODE	 = 0x40000000,

/*
 * Indicates that this surface will be an optimized surface. This flag is
 * currently only valid in conjunction with the DDSCAPS_TEXTURE flag. The surface
 * will be created without any underlying video memory until loaded.
 */
DDSCAPS_OPTIMIZED			= 0x80000000,

}

enum : uint {

 /****************************************************************************
 *
 * DIRECTDRAW DRIVER CAPABILITY FLAGS
 *
 ****************************************************************************/

/*
 * Display hardware has 3D acceleration.
 */
DDCAPS_3D			= 0x00000001,

/*
 * Indicates that DirectDraw will support only dest rectangles that are aligned
 * on DIRECTDRAWCAPS.dwAlignBoundaryDest boundaries of the surface, respectively.
 * READ ONLY.
 */
DDCAPS_ALIGNBOUNDARYDEST	 = 0x00000002,

/*
 * Indicates that DirectDraw will support only source rectangles  whose sizes in
 * BYTEs are DIRECTDRAWCAPS.dwAlignSizeDest multiples, respectively.  READ ONLY.
 */
DDCAPS_ALIGNSIZEDEST		 = 0x00000004,
/*
 * Indicates that DirectDraw will support only source rectangles that are aligned
 * on DIRECTDRAWCAPS.dwAlignBoundarySrc boundaries of the surface, respectively.
 * READ ONLY.
 */
DDCAPS_ALIGNBOUNDARYSRC		 = 0x00000008,

/*
 * Indicates that DirectDraw will support only source rectangles  whose sizes in
 * BYTEs are DIRECTDRAWCAPS.dwAlignSizeSrc multiples, respectively.  READ ONLY.
 */
DDCAPS_ALIGNSIZESRC		 = 0x00000010,

/*
 * Indicates that DirectDraw will create video memory surfaces that have a stride
 * alignment equal to DIRECTDRAWCAPS.dwAlignStride.  READ ONLY.
 */
DDCAPS_ALIGNSTRIDE		 = 0x00000020,

/*
 * Display hardware is capable of blt operations.
 */
DDCAPS_BLT			 = 0x00000040,

/*
 * Display hardware is capable of asynchronous blt operations.
 */
DDCAPS_BLTQUEUE			 = 0x00000080,

/*
 * Display hardware is capable of color space conversions during the blt operation.
 */
DDCAPS_BLTFOURCC		 = 0x00000100,

/*
 * Display hardware is capable of stretching during blt operations.
 */
DDCAPS_BLTSTRETCH		 = 0x00000200,

/*
 * Display hardware is shared with GDI.
 */
DDCAPS_GDI			= 0x00000400,

/*
 * Display hardware can overlay.
 */
DDCAPS_OVERLAY			 = 0x00000800,

/*
 * Set if display hardware supports overlays but can not clip them.
 */
DDCAPS_OVERLAYCANTCLIP		 = 0x00001000,

/*
 * Indicates that overlay hardware is capable of color space conversions during
 * the overlay operation.
 */
DDCAPS_OVERLAYFOURCC		 = 0x00002000,

/*
 * Indicates that stretching can be done by the overlay hardware.
 */
DDCAPS_OVERLAYSTRETCH		 = 0x00004000,

/*
 * Indicates that unique DirectDrawPalettes can be created for DirectDrawSurfaces
 * other than the primary surface.
 */
DDCAPS_PALETTE			 = 0x00008000,

/*
 * Indicates that palette changes can be syncd with the veritcal refresh.
 */
DDCAPS_PALETTEVSYNC		 = 0x00010000,

/*
 * Display hardware can return the current scan line.
 */
DDCAPS_READSCANLINE		 = 0x00020000,

/*
 * Display hardware has stereo vision capabilities.  DDSCAPS_PRIMARYSURFACELEFT
 * can be created.
 */
DDCAPS_STEREOVIEW		 = 0x00040000,

/*
 * Display hardware is capable of generating a vertical blank interrupt.
 */
DDCAPS_VBI			 = 0x00080000,

/*
 * Supports the use of z buffers with blt operations.
 */
DDCAPS_ZBLTS			 = 0x00100000,

/*
 * Supports Z Ordering of overlays.
 */
DDCAPS_ZOVERLAYS		 = 0x00200000,

/*
 * Supports color key
 */
DDCAPS_COLORKEY			 = 0x00400000,

/*
 * Supports alpha surfaces
 */
DDCAPS_ALPHA			 = 0x00800000,

/*
 * colorkey is hardware assisted(DDCAPS_COLORKEY will also be set)
 */
DDCAPS_COLORKEYHWASSIST		 = 0x01000000,

/*
 * no hardware support at all
 */
DDCAPS_NOHARDWARE		 = 0x02000000,

/*
 * Display hardware is capable of color fill with bltter
 */
DDCAPS_BLTCOLORFILL		 = 0x04000000,

/*
 * Display hardware is bank switched, and potentially very slow at
 * random access to VRAM.
 */
DDCAPS_BANKSWITCHED		 = 0x08000000,

/*
 * Display hardware is capable of depth filling Z-buffers with bltter
 */
DDCAPS_BLTDEPTHFILL		 = 0x10000000,

/*
 * Display hardware is capable of clipping while bltting.
 */
DDCAPS_CANCLIP			 = 0x20000000,

/*
 * Display hardware is capable of clipping while stretch bltting.
 */
DDCAPS_CANCLIPSTRETCHED		 = 0x40000000,

/*
 * Display hardware is capable of bltting to or from system memory
 */
DDCAPS_CANBLTSYSMEM		 = 0x80000000,

}

enum : uint {
 /****************************************************************************
 *
 * MORE DIRECTDRAW DRIVER CAPABILITY FLAGS (dwCaps2)
 *
 ****************************************************************************/

/*
 * Display hardware is certified
 */
 DDCAPS2_CERTIFIED		=0x00000001,

/*
 * Driver cannot interleave 2D operations (lock and blt) to surfaces with
 * Direct3D rendering operations between calls to BeginScene() and EndScene()
 */
 DDCAPS2_NO2DDURING3DSCENE       =0x00000002,

/*
 * Display hardware contains a video port
 */
 DDCAPS2_VIDEOPORT		=0x00000004,

/*
 * The overlay can be automatically flipped according to the video port
 * VSYNCs, providing automatic doubled buffered display of video port
 * data using an overlay
 */
 DDCAPS2_AUTOFLIPOVERLAY		=0x00000008,

/*
 * Overlay can display each field of interlaced data individually while
 * it is interleaved in memory without causing jittery artifacts.
 */
 DDCAPS2_CANBOBINTERLEAVED	=0x00000010,

/*
 * Overlay can display each field of interlaced data individually while
 * it is not interleaved in memory without causing jittery artifacts.
 */
 DDCAPS2_CANBOBNONINTERLEAVED	=0x00000020,

/*
 * The overlay surface contains color controls (brightness, sharpness, etc.)
 */
 DDCAPS2_COLORCONTROLOVERLAY	=0x00000040,

/*
 * The primary surface contains color controls (gamma, etc.)
 */
 DDCAPS2_COLORCONTROLPRIMARY	=0x00000080,

/*
 * RGBZ -> RGB supported for 16:16 RGB:Z
 */
 DDCAPS2_CANDROPZ16BIT		=0x00000100,

/*
 * Driver supports non-local video memory.
 */
 DDCAPS2_NONLOCALVIDMEM          =0x00000200,

/*
 * Dirver supports non-local video memory but has different capabilities for
 * non-local video memory surfaces. If this bit is set then so must
 * DDCAPS2_NONLOCALVIDMEM.
 */
 DDCAPS2_NONLOCALVIDMEMCAPS      =0x00000400,

/*
 * Driver neither requires nor prefers surfaces to be pagelocked when performing
 * blts involving system memory surfaces
 */
 DDCAPS2_NOPAGELOCKREQUIRED      =0x00000800,

/*
 * Driver can create surfaces which are wider than the primary surface
 */
 DDCAPS2_WIDESURFACES            =0x00001000,

/*
 * Driver supports bob without using a video port by handling the
 * DDFLIP_ODD and DDFLIP_EVEN flags specified in Flip.
 */
 DDCAPS2_CANFLIPODDEVEN          =0x00002000,

/*
 * Driver supports bob using hardware
 */
 DDCAPS2_CANBOBHARDWARE          =0x00004000,

/*
 * Driver supports bltting any FOURCC surface to another surface of the same FOURCC
 */
 DDCAPS2_COPYFOURCC              =0x00008000,


/****************************************************************************
 *
 * DIRECTDRAW FX ALPHA CAPABILITY FLAGS
 *
 ****************************************************************************/

/*
 * Supports alpha blending around the edge of a source color keyed surface.
 * For Blt.
 */
 DDFXALPHACAPS_BLTALPHAEDGEBLEND		=0x00000001,

/*
 * Supports alpha information in the pixel format.  The bit depth of alpha
 * information in the pixel format can be 1,2,4, or 8.	The alpha value becomes
 * more opaque as the alpha value increases.  (0 is transparent.)
 * For Blt.
 */
 DDFXALPHACAPS_BLTALPHAPIXELS		=0x00000002,

/*
 * Supports alpha information in the pixel format.  The bit depth of alpha
 * information in the pixel format can be 1,2,4, or 8.	The alpha value
 * becomes more transparent as the alpha value increases.  (0 is opaque.)
 * This flag can only be set if DDCAPS_ALPHA is set.
 * For Blt.
 */
 DDFXALPHACAPS_BLTALPHAPIXELSNEG		=0x00000004,

/*
 * Supports alpha only surfaces.  The bit depth of an alpha only surface can be
 * 1,2,4, or 8.	 The alpha value becomes more opaque as the alpha value increases.
 * (0 is transparent.)
 * For Blt.
 */
 DDFXALPHACAPS_BLTALPHASURFACES		=0x00000008,

/*
 * The depth of the alpha channel data can range can be 1,2,4, or 8.
 * The NEG suffix indicates that this alpha channel becomes more transparent
 * as the alpha value increases. (0 is opaque.)	 This flag can only be set if
 * DDCAPS_ALPHA is set.
 * For Blt.
 */
 DDFXALPHACAPS_BLTALPHASURFACESNEG	=0x00000010,

/*
 * Supports alpha blending around the edge of a source color keyed surface.
 * For Overlays.
 */
 DDFXALPHACAPS_OVERLAYALPHAEDGEBLEND	=0x00000020,

/*
 * Supports alpha information in the pixel format.  The bit depth of alpha
 * information in the pixel format can be 1,2,4, or 8.	The alpha value becomes
 * more opaque as the alpha value increases.  (0 is transparent.)
 * For Overlays.
 */
 DDFXALPHACAPS_OVERLAYALPHAPIXELS	=0x00000040,

/*
 * Supports alpha information in the pixel format.  The bit depth of alpha
 * information in the pixel format can be 1,2,4, or 8.	The alpha value
 * becomes more transparent as the alpha value increases.  (0 is opaque.)
 * This flag can only be set if DDCAPS_ALPHA is set.
 * For Overlays.
 */
 DDFXALPHACAPS_OVERLAYALPHAPIXELSNEG	=0x00000080,

/*
 * Supports alpha only surfaces.  The bit depth of an alpha only surface can be
 * 1,2,4, or 8.	 The alpha value becomes more opaque as the alpha value increases.
 * (0 is transparent.)
 * For Overlays.
 */
 DDFXALPHACAPS_OVERLAYALPHASURFACES	=0x00000100,

/*
 * The depth of the alpha channel data can range can be 1,2,4, or 8.
 * The NEG suffix indicates that this alpha channel becomes more transparent
 * as the alpha value increases. (0 is opaque.)	 This flag can only be set if
 * DDCAPS_ALPHA is set.
 * For Overlays.
 */
 DDFXALPHACAPS_OVERLAYALPHASURFACESNEG	=0x00000200,

/****************************************************************************
 *
 * DIRECTDRAW FX CAPABILITY FLAGS
 *
 ****************************************************************************/

/*
 * Uses arithmetic operations to stretch and shrink surfaces during blt
 * rather than pixel doubling techniques.  Along the Y axis.
 */
 DDFXCAPS_BLTARITHSTRETCHY	=0x00000020,

/*
 * Uses arithmetic operations to stretch during blt
 * rather than pixel doubling techniques.  Along the Y axis. Only
 * works for x1, x2, etc.
 */
 DDFXCAPS_BLTARITHSTRETCHYN	=0x00000010,

/*
 * Supports mirroring left to right in blt.
 */
 DDFXCAPS_BLTMIRRORLEFTRIGHT	=0x00000040,

/*
 * Supports mirroring top to bottom in blt.
 */
 DDFXCAPS_BLTMIRRORUPDOWN	=0x00000080,

/*
 * Supports arbitrary rotation for blts.
 */
 DDFXCAPS_BLTROTATION		=0x00000100,

/*
 * Supports 90 degree rotations for blts.
 */
 DDFXCAPS_BLTROTATION90		=0x00000200,

/*
 * DirectDraw supports arbitrary shrinking of a surface along the
 * x axis (horizontal direction) for blts.
 */
 DDFXCAPS_BLTSHRINKX		=0x00000400,

/*
 * DirectDraw supports integer shrinking (1x,2x,) of a surface
 * along the x axis (horizontal direction) for blts.
 */
 DDFXCAPS_BLTSHRINKXN		=0x00000800,

/*
 * DirectDraw supports arbitrary shrinking of a surface along the
 * y axis (horizontal direction) for blts.
 */
 DDFXCAPS_BLTSHRINKY		=0x00001000,

/*
 * DirectDraw supports integer shrinking (1x,2x,) of a surface
 * along the y axis (vertical direction) for blts.
 */
 DDFXCAPS_BLTSHRINKYN		=0x00002000,

/*
 * DirectDraw supports arbitrary stretching of a surface along the
 * x axis (horizontal direction) for blts.
 */
 DDFXCAPS_BLTSTRETCHX		=0x00004000,

/*
 * DirectDraw supports integer stretching (1x,2x,) of a surface
 * along the x axis (horizontal direction) for blts.
 */
 DDFXCAPS_BLTSTRETCHXN		=0x00008000,

/*
 * DirectDraw supports arbitrary stretching of a surface along the
 * y axis (horizontal direction) for blts.
 */
 DDFXCAPS_BLTSTRETCHY		=0x00010000,

/*
 * DirectDraw supports integer stretching (1x,2x,) of a surface
 * along the y axis (vertical direction) for blts.
 */
 DDFXCAPS_BLTSTRETCHYN		=0x00020000,

/*
 * Uses arithmetic operations to stretch and shrink surfaces during
 * overlay rather than pixel doubling techniques.  Along the Y axis
 * for overlays.
 */
 DDFXCAPS_OVERLAYARITHSTRETCHY=0x00040000,

/*
 * Uses arithmetic operations to stretch surfaces during
 * overlay rather than pixel doubling techniques.  Along the Y axis
 * for overlays. Only works for x1, x2, etc.
 */
 DDFXCAPS_OVERLAYARITHSTRETCHYN	=0x00000008,

/*
 * DirectDraw supports arbitrary shrinking of a surface along the
 * x axis (horizontal direction) for overlays.
 */
 DDFXCAPS_OVERLAYSHRINKX		=0x00080000,

/*
 * DirectDraw supports integer shrinking (1x,2x,) of a surface
 * along the x axis (horizontal direction) for overlays.
 */
 DDFXCAPS_OVERLAYSHRINKXN	=0x00100000,

/*
 * DirectDraw supports arbitrary shrinking of a surface along the
 * y axis (horizontal direction) for overlays.
 */
 DDFXCAPS_OVERLAYSHRINKY		=0x00200000,

/*
 * DirectDraw supports integer shrinking (1x,2x,) of a surface
 * along the y axis (vertical direction) for overlays.
 */
 DDFXCAPS_OVERLAYSHRINKYN	=0x00400000,

/*
 * DirectDraw supports arbitrary stretching of a surface along the
 * x axis (horizontal direction) for overlays.
 */
 DDFXCAPS_OVERLAYSTRETCHX	=0x00800000,

/*
 * DirectDraw supports integer stretching (1x,2x,) of a surface
 * along the x axis (horizontal direction) for overlays.
 */
 DDFXCAPS_OVERLAYSTRETCHXN	=0x01000000,

/*
 * DirectDraw supports arbitrary stretching of a surface along the
 * y axis (horizontal direction) for overlays.
 */
 DDFXCAPS_OVERLAYSTRETCHY	=0x02000000,

/*
 * DirectDraw supports integer stretching (1x,2x,) of a surface
 * along the y axis (vertical direction) for overlays.
 */
 DDFXCAPS_OVERLAYSTRETCHYN	=0x04000000,

/*
 * DirectDraw supports mirroring of overlays across the vertical axis
 */
 DDFXCAPS_OVERLAYMIRRORLEFTRIGHT	=0x08000000,

/*
 * DirectDraw supports mirroring of overlays across the horizontal axis
 */
 DDFXCAPS_OVERLAYMIRRORUPDOWN	=0x10000000,

/****************************************************************************
 *
 * DIRECTDRAW STEREO VIEW CAPABILITIES
 *
 ****************************************************************************/

/*
 * The stereo view is accomplished via enigma encoding.
 */
 DDSVCAPS_ENIGMA			=0x00000001,

/*
 * The stereo view is accomplished via high frequency flickering.
 */
 DDSVCAPS_FLICKER		=0x00000002,

/*
 * The stereo view is accomplished via red and blue filters applied
 * to the left and right eyes.	All images must adapt their colorspaces
 * for this process.
 */
 DDSVCAPS_REDBLUE		=0x00000004,

/*
 * The stereo view is accomplished with split screen technology.
 */
 DDSVCAPS_SPLIT			=0x00000008,

/****************************************************************************
 *
 * DIRECTDRAWPALETTE CAPABILITIES
 *
 ****************************************************************************/

/*
 * Index is 4 bits.  There are sixteen color entries in the palette table.
 */
 DDPCAPS_4BIT			=0x00000001,

/*
 * Index is onto a 8 bit color index.  This field is only valid with the
 * DDPCAPS_1BIT, DDPCAPS_2BIT or DDPCAPS_4BIT capability and the target
 * surface is in 8bpp. Each color entry is one byte long and is an index
 * into destination surface's 8bpp palette.
 */
 DDPCAPS_8BITENTRIES		=0x00000002,

/*
 * Index is 8 bits.  There are 256 color entries in the palette table.
 */
 DDPCAPS_8BIT			=0x00000004,

/*
 * Indicates that this DIRECTDRAWPALETTE should use the palette color array
 * passed into the lpDDColorArray parameter to initialize the DIRECTDRAWPALETTE
 * object.
 */
 DDPCAPS_INITIALIZE		=0x00000008,

/*
 * This palette is the one attached to the primary surface.  Changing this
 * table has immediate effect on the display unless DDPSETPAL_VSYNC is specified
 * and supported.
 */
 DDPCAPS_PRIMARYSURFACE		=0x00000010,

/*
 * This palette is the one attached to the primary surface left.  Changing
 * this table has immediate effect on the display for the left eye unless
 * DDPSETPAL_VSYNC is specified and supported.
 */
 DDPCAPS_PRIMARYSURFACELEFT	=0x00000020,

/*
 * This palette can have all 256 entries defined
 */
 DDPCAPS_ALLOW256		=0x00000040,

/*
 * This palette can have modifications to it synced with the monitors
 * refresh rate.
 */
 DDPCAPS_VSYNC			=0x00000080,

/*
 * Index is 1 bit.  There are two color entries in the palette table.
 */
 DDPCAPS_1BIT			=0x00000100,

/*
 * Index is 2 bit.  There are four color entries in the palette table.
 */
 DDPCAPS_2BIT			=0x00000200,


/****************************************************************************
 *
 * DIRECTDRAWPALETTE SETENTRY CONSTANTS
 *
 ****************************************************************************/


/****************************************************************************
 *
 * DIRECTDRAWPALETTE GETENTRY CONSTANTS
 *
 ****************************************************************************/

/* 0 is the only legal value */

/****************************************************************************
 *
 * DIRECTDRAWSURFACE SETPALETTE CONSTANTS
 *
 ****************************************************************************/


/****************************************************************************
 *
 * DIRECTDRAW BITDEPTH CONSTANTS
 *
 * NOTE:  These are only used to indicate supported bit depths.   These
 * are flags only, they are not to be used as an actual bit depth.   The
 * absolute numbers 1, 2, 4, 8, 16, 24 and 32 are used to indicate actual
 * bit depths in a surface or for changing the display mode.
 *
 ****************************************************************************/

/*
 * 1 bit per pixel.
 */
 DDBD_1			=0x00004000,

/*
 * 2 bits per pixel.
 */
 DDBD_2			=0x00002000,

/*
 * 4 bits per pixel.
 */
 DDBD_4			=0x00001000,

/*
 * 8 bits per pixel.
 */
 DDBD_8			=0x00000800,

/*
 * 16 bits per pixel.
 */
 DDBD_16			=0x00000400,

/*
 * 24 bits per pixel.
 */
 DDBD_24			=0X00000200,

/*
 * 32 bits per pixel.
 */
 DDBD_32			=0x00000100,

/****************************************************************************
 *
 * DIRECTDRAWSURFACE SET/GET COLOR KEY FLAGS
 *
 ****************************************************************************/

/*
 * Set if the structure contains a color space.	 Not set if the structure
 * contains a single color key.
 */
 DDCKEY_COLORSPACE	=0x00000001,

/*
 * Set if the structure specifies a color key or color space which is to be
 * used as a destination color key for blt operations.
 */
 DDCKEY_DESTBLT		=0x00000002,

/*
 * Set if the structure specifies a color key or color space which is to be
 * used as a destination color key for overlay operations.
 */
 DDCKEY_DESTOVERLAY	=0x00000004,

/*
 * Set if the structure specifies a color key or color space which is to be
 * used as a source color key for blt operations.
 */
 DDCKEY_SRCBLT		=0x00000008,

/*
 * Set if the structure specifies a color key or color space which is to be
 * used as a source color key for overlay operations.
 */
 DDCKEY_SRCOVERLAY	=0x00000010,


/****************************************************************************
 *
 * DIRECTDRAW COLOR KEY CAPABILITY FLAGS
 *
 ****************************************************************************/

/*
 * Supports transparent blting using a color key to identify the replaceable
 * bits of the destination surface for RGB colors.
 */
 DDCKEYCAPS_DESTBLT			=0x00000001,

/*
 * Supports transparent blting using a color space to identify the replaceable
 * bits of the destination surface for RGB colors.
 */
 DDCKEYCAPS_DESTBLTCLRSPACE		=0x00000002,

/*
 * Supports transparent blting using a color space to identify the replaceable
 * bits of the destination surface for YUV colors.
 */
 DDCKEYCAPS_DESTBLTCLRSPACEYUV		=0x00000004,

/*
 * Supports transparent blting using a color key to identify the replaceable
 * bits of the destination surface for YUV colors.
 */
 DDCKEYCAPS_DESTBLTYUV			=0x00000008,

/*
 * Supports overlaying using colorkeying of the replaceable bits of the surface
 * being overlayed for RGB colors.
 */
 DDCKEYCAPS_DESTOVERLAY			=0x00000010,

/*
 * Supports a color space as the color key for the destination for RGB colors.
 */
 DDCKEYCAPS_DESTOVERLAYCLRSPACE		=0x00000020,

/*
 * Supports a color space as the color key for the destination for YUV colors.
 */
 DDCKEYCAPS_DESTOVERLAYCLRSPACEYUV	=0x00000040,

/*
 * Supports only one active destination color key value for visible overlay
 * surfaces.
 */
 DDCKEYCAPS_DESTOVERLAYONEACTIVE		=0x00000080,

/*
 * Supports overlaying using colorkeying of the replaceable bits of the
 * surface being overlayed for YUV colors.
 */
 DDCKEYCAPS_DESTOVERLAYYUV		=0x00000100,

/*
 * Supports transparent blting using the color key for the source with
 * this surface for RGB colors.
 */
 DDCKEYCAPS_SRCBLT			=0x00000200,

/*
 * Supports transparent blting using a color space for the source with
 * this surface for RGB colors.
 */
 DDCKEYCAPS_SRCBLTCLRSPACE		=0x00000400,

/*
 * Supports transparent blting using a color space for the source with
 * this surface for YUV colors.
 */
 DDCKEYCAPS_SRCBLTCLRSPACEYUV		=0x00000800,

/*
 * Supports transparent blting using the color key for the source with
 * this surface for YUV colors.
 */
 DDCKEYCAPS_SRCBLTYUV			=0x00001000,

/*
 * Supports overlays using the color key for the source with this
 * overlay surface for RGB colors.
 */
 DDCKEYCAPS_SRCOVERLAY			=0x00002000,

/*
 * Supports overlays using a color space as the source color key for
 * the overlay surface for RGB colors.
 */
 DDCKEYCAPS_SRCOVERLAYCLRSPACE		=0x00004000,

/*
 * Supports overlays using a color space as the source color key for
 * the overlay surface for YUV colors.
 */
 DDCKEYCAPS_SRCOVERLAYCLRSPACEYUV	=0x00008000,

/*
 * Supports only one active source color key value for visible
 * overlay surfaces.
 */
 DDCKEYCAPS_SRCOVERLAYONEACTIVE		=0x00010000,

/*
 * Supports overlays using the color key for the source with this
 * overlay surface for YUV colors.
 */
 DDCKEYCAPS_SRCOVERLAYYUV		=0x00020000,

/*
 * there are no bandwidth trade-offs for using colorkey with an overlay
 */
 DDCKEYCAPS_NOCOSTOVERLAY		=0x00040000,


/****************************************************************************
 *
 * DIRECTDRAW PIXELFORMAT FLAGS
 *
 ****************************************************************************/

/*
 * The surface has alpha channel information in the pixel format.
 */
 DDPF_ALPHAPIXELS			=0x00000001,

/*
 * The pixel format contains alpha only information
 */
 DDPF_ALPHA				=0x00000002,

/*
 * The FourCC code is valid.
 */
 DDPF_FOURCC				=0x00000004,

/*
 * The surface is 4-bit color indexed.
 */
 DDPF_PALETTEINDEXED4			=0x00000008,

/*
 * The surface is indexed into a palette which stores indices
 * into the destination surface's 8-bit palette.
 */
 DDPF_PALETTEINDEXEDTO8			=0x00000010,

/*
 * The surface is 8-bit color indexed.
 */
 DDPF_PALETTEINDEXED8			=0x00000020,

/*
 * The RGB data in the pixel format structure is valid.
 */
 DDPF_RGB				=0x00000040,

/*
 * The surface will accept pixel data in the format specified
 * and compress it during the write.
 */
 DDPF_COMPRESSED				=0x00000080,

/*
 * The surface will accept RGB data and translate it during
 * the write to YUV data.  The format of the data to be written
 * will be contained in the pixel format structure.  The DDPF_RGB
 * flag will be set.
 */
 DDPF_RGBTOYUV				=0x00000100,

/*
 * pixel format is YUV - YUV data in pixel format struct is valid
 */
 DDPF_YUV				=0x00000200,

/*
 * pixel format is a z buffer only surface
 */
 DDPF_ZBUFFER				=0x00000400,

/*
 * The surface is 1-bit color indexed.
 */
 DDPF_PALETTEINDEXED1			=0x00000800,

/*
 * The surface is 2-bit color indexed.
 */
 DDPF_PALETTEINDEXED2			= 0x00001000,

/*
 * The surface contains Z information in the pixels
 */
 DDPF_ZPIXELS				= 0x00002000

}

enum : uint {
	
/*
 * ddsCaps field is valid.
 */
DDSD_CAPS		 = 0x00000001,	// default

/*
 * dwHeight field is valid.
 */
DDSD_HEIGHT		 = 0x00000002,

/*
 * dwWidth field is valid.
 */
DDSD_WIDTH		 = 0x00000004,

/*
 * lPitch is valid.
 */
DDSD_PITCH		 = 0x00000008,

/*
 * dwBackBufferCount is valid.
 */
DDSD_BACKBUFFERCOUNT	 = 0x00000020,

/*
 * dwZBufferBitDepth is valid.
 */
DDSD_ZBUFFERBITDEPTH	 = 0x00000040,

/*
 * dwAlphaBitDepth is valid.
 */
DDSD_ALPHABITDEPTH	 = 0x00000080,


/*
 * lpSurface is valid.
 */
DDSD_LPSURFACE		 = 0x00000800,

/*
 * ddpfPixelFormat is valid.
 */
DDSD_PIXELFORMAT	 = 0x00001000,

/*
 * ddckCKDestOverlay is valid.
 */
DDSD_CKDESTOVERLAY	 = 0x00002000,

/*
 * ddckCKDestBlt is valid.
 */
DDSD_CKDESTBLT		 = 0x00004000,

/*
 * ddckCKSrcOverlay is valid.
 */
DDSD_CKSRCOVERLAY	 = 0x00008000,

/*
 * ddckCKSrcBlt is valid.
 */
DDSD_CKSRCBLT		 = 0x00010000,

/*
 * dwMipMapCount is valid.
 */
DDSD_MIPMAPCOUNT         = 0x00020000,

 /*
  * dwRefreshRate is valid
  */
DDSD_REFRESHRATE	 = 0x00040000,

/*
 * dwLinearSize is valid
 */
DDSD_LINEARSIZE		= 0x00080000,

/*
 * All input fields are valid.
 */
DDSD_ALL		 = 0x000ff9ee
}

/****************************************************************************
 *
 * DIRECTDRAW ENUMCALLBACK RETURN VALUES
 *
 * EnumCallback returns are used to control the flow of the DIRECTDRAW and
 * DIRECTDRAWSURFACE object enumerations.   They can only be returned by
 * enumeration callback routines.
 *
 ****************************************************************************/

enum {
/*
 * stop the enumeration
 */
DDENUMRET_CANCEL			 = 0,

/*
 * continue the enumeration
 */
DDENUMRET_OK				 = 1
}

/****************************************************************************
 *
 * DIRECTDRAW ERRORS
 *
 * Errors are represented by negative values and cannot be combined.
 *
 ****************************************************************************/


/*
#define _FACDD	0x876
#define =(0x88760000|( code )	MAKE_HRESULT( 1, _FACDD, code )
#define MAKE_HRESULT(sev,fac,code) \
    ((HRESULT) (((unsigned long)(sev)<<31) | ((unsigned long)(fac)<<16) | ((unsigned long)(code))) )
//  = (0x88760000|(code));

*/
enum : uint {
/*
 * This object is already initialized
 */
DDERR_ALREADYINITIALIZED		=(0x88760000|( 5 )),

/*
 * This surface can not be attached to the requested surface.
 */
 DDERR_CANNOTATTACHSURFACE		=(0x88760000|( 10 )),

/*
 * This surface can not be detached from the requested surface.
 */
 DDERR_CANNOTDETACHSURFACE		=(0x88760000|( 20 )),

/*
 * Support is currently not available.
 */
 DDERR_CURRENTLYNOTAVAIL			=(0x88760000|( 40 )),

/*
 * An exception was encountered while performing the requested operation
 */
 DDERR_EXCEPTION				=(0x88760000|( 55 )),

/*
 * Generic failure.
 */
 DDERR_GENERIC				=cast(uint)E_FAIL,

/*
 * Height of rectangle provided is not a multiple of reqd alignment
 */
 DDERR_HEIGHTALIGN			=(0x88760000|( 90 )),

/*
 * Unable to match primary surface creation request with existing
 * primary surface.
 */
 DDERR_INCOMPATIBLEPRIMARY		=(0x88760000|( 95 )),

/*
 * One or more of the caps bits passed to the callback are incorrect.
 */
 DDERR_INVALIDCAPS			=(0x88760000|( 100 )),

/*
 * DirectDraw does not support provided Cliplist.
 */
 DDERR_INVALIDCLIPLIST			=(0x88760000|( 110 )),

/*
 * DirectDraw does not support the requested mode
 */
 DDERR_INVALIDMODE			=(0x88760000|( 120 )),

/*
 * DirectDraw received a pointer that was an invalid DIRECTDRAW object.
 */
 DDERR_INVALIDOBJECT			=(0x88760000|( 130 )),

/*
 * One or more of the parameters passed to the callback function are
 * incorrect.
 */
 DDERR_INVALIDPARAMS			=cast(uint)E_INVALIDARG,

/*
 * pixel format was invalid as specified
 */
 DDERR_INVALIDPIXELFORMAT		=(0x88760000|( 145 )),

/*
 * Rectangle provided was invalid.
 */
 DDERR_INVALIDRECT			=(0x88760000|( 150 )),

/*
 * Operation could not be carried out because one or more surfaces are locked
 */
 DDERR_LOCKEDSURFACES			=(0x88760000|( 160 )),

/*
 * There is no 3D present.
 */
 DDERR_NO3D				=(0x88760000|( 170 )),

/*
 * Operation could not be carried out because there is no alpha accleration
 * hardware present or available.
 */
 DDERR_NOALPHAHW				=(0x88760000|( 180 )),


/*
 * no clip list available
 */
 DDERR_NOCLIPLIST			=(0x88760000|( 205 )),

/*
 * Operation could not be carried out because there is no color conversion
 * hardware present or available.
 */
 DDERR_NOCOLORCONVHW			=(0x88760000|( 210 )),

/*
 * Create function called without DirectDraw object method SetCooperativeLevel
 * being called.
 */
 DDERR_NOCOOPERATIVELEVELSET		=(0x88760000|( 212 )),

/*
 * Surface doesn't currently have a color key
 */
 DDERR_NOCOLORKEY			=(0x88760000|( 215 )),

/*
 * Operation could not be carried out because there is no hardware support
 * of the dest color key.
 */
 DDERR_NOCOLORKEYHW			=(0x88760000|( 220 )),

/*
 * No DirectDraw support possible with current display driver
 */
 DDERR_NODIRECTDRAWSUPPORT		=(0x88760000|( 222 )),

/*
 * Operation requires the application to have exclusive mode but the
 * application does not have exclusive mode.
 */
 DDERR_NOEXCLUSIVEMODE			=(0x88760000|( 225 )),

/*
 * Flipping visible surfaces is not supported.
 */
 DDERR_NOFLIPHW				=(0x88760000|( 230 )),

/*
 * There is no GDI present.
 */
 DDERR_NOGDI				=(0x88760000|( 240 )),

/*
 * Operation could not be carried out because there is no hardware present
 * or available.
 */
 DDERR_NOMIRRORHW			=(0x88760000|( 250 )),

/*
 * Requested item was not found
 */
 DDERR_NOTFOUND				=(0x88760000|( 255 )),

/*
 * Operation could not be carried out because there is no overlay hardware
 * present or available.
 */
 DDERR_NOOVERLAYHW			=(0x88760000|( 260 )),

/*
 * Operation could not be carried out because there is no appropriate raster
 * op hardware present or available.
 */
 DDERR_NORASTEROPHW			=(0x88760000|( 280 )),

/*
 * Operation could not be carried out because there is no rotation hardware
 * present or available.
 */
 DDERR_NOROTATIONHW			=(0x88760000|( 290 )),

/*
 * Operation could not be carried out because there is no hardware support
 * for stretching
 */
 DDERR_NOSTRETCHHW			=(0x88760000|( 310 )),

/*
 * DirectDrawSurface is not in 4 bit color palette and the requested operation
 * requires 4 bit color palette.
 */
 DDERR_NOT4BITCOLOR			=(0x88760000|( 316 )),

/*
 * DirectDrawSurface is not in 4 bit color index palette and the requested
 * operation requires 4 bit color index palette.
 */
 DDERR_NOT4BITCOLORINDEX			=(0x88760000|( 317 )),

/*
 * DirectDraw Surface is not in 8 bit color mode and the requested operation
 * requires 8 bit color.
 */
 DDERR_NOT8BITCOLOR			=(0x88760000|( 320 )),

/*
 * Operation could not be carried out because there is no texture mapping
 * hardware present or available.
 */
 DDERR_NOTEXTUREHW			=(0x88760000|( 330 )),

/*
 * Operation could not be carried out because there is no hardware support
 * for vertical blank synchronized operations.
 */
 DDERR_NOVSYNCHW				=(0x88760000|( 335 )),

/*
 * Operation could not be carried out because there is no hardware support
 * for zbuffer blting.
 */
 DDERR_NOZBUFFERHW			=(0x88760000|( 340 )),

/*
 * Overlay surfaces could not be z layered based on their BltOrder because
 * the hardware does not support z layering of overlays.
 */
 DDERR_NOZOVERLAYHW			=(0x88760000|( 350 )),

/*
 * The hardware needed for the requested operation has already been
 * allocated.
 */
 DDERR_OUTOFCAPS				=(0x88760000|( 360 )),

/*
 * DirectDraw does not have enough memory to perform the operation.
 */
 DDERR_OUTOFMEMORY			=cast(uint)E_OUTOFMEMORY,

/*
 * DirectDraw does not have enough memory to perform the operation.
 */
 DDERR_OUTOFVIDEOMEMORY			=(0x88760000|( 380 )),

/*
 * hardware does not support clipped overlays
 */
 DDERR_OVERLAYCANTCLIP			=(0x88760000|( 382 )),

/*
 * Can only have ony color key active at one time for overlays
 */
 DDERR_OVERLAYCOLORKEYONLYONEACTIVE	=(0x88760000|( 384 )),

/*
 * Access to this palette is being refused because the palette is already
 * locked by another thread.
 */
 DDERR_PALETTEBUSY			=(0x88760000|( 387 )),

/*
 * No src color key specified for this operation.
 */
 DDERR_COLORKEYNOTSET			=(0x88760000|( 400 )),

/*
 * This surface is already attached to the surface it is being attached to.
 */
 DDERR_SURFACEALREADYATTACHED		=(0x88760000|( 410 )),

/*
 * This surface is already a dependency of the surface it is being made a
 * dependency of.
 */
 DDERR_SURFACEALREADYDEPENDENT		=(0x88760000|( 420 )),

/*
 * Access to this surface is being refused because the surface is already
 * locked by another thread.
 */
 DDERR_SURFACEBUSY			=(0x88760000|( 430 )),

/*
 * Access to this surface is being refused because no driver exists
 * which can supply a pointer to the surface.
 * This is most likely to happen when attempting to lock the primary
 * surface when no DCI provider is present.
 * Will also happen on attempts to lock an optimized surface.
 */
 DDERR_CANTLOCKSURFACE                   =(0x88760000|( 435 )),

/*
 * Access to Surface refused because Surface is obscured.
 */
 DDERR_SURFACEISOBSCURED			=(0x88760000|( 440 )),

/*
 * Access to this surface is being refused because the surface is gone.
 * The DIRECTDRAWSURFACE object representing this surface should
 * have Restore called on it.
 */
 DDERR_SURFACELOST			=(0x88760000|( 450 )),

/*
 * The requested surface is not attached.
 */
 DDERR_SURFACENOTATTACHED		=(0x88760000|( 460 )),

/*
 * Height requested by DirectDraw is too large.
 */
 DDERR_TOOBIGHEIGHT			=(0x88760000|( 470 )),

/*
 * Size requested by DirectDraw is too large --	 The individual height and
 * width are OK.
 */
 DDERR_TOOBIGSIZE			=(0x88760000|( 480 )),

/*
 * Width requested by DirectDraw is too large.
 */
 DDERR_TOOBIGWIDTH			=(0x88760000|( 490 )),

/*
 * Action not supported.
 */
 DDERR_UNSUPPORTED			=cast(uint)E_NOTIMPL,

/*
 * FOURCC format requested is unsupported by DirectDraw
 */
 DDERR_UNSUPPORTEDFORMAT			=(0x88760000|( 510 )),

/*
 * Bitmask in the pixel format requested is unsupported by DirectDraw
 */
 DDERR_UNSUPPORTEDMASK			=(0x88760000|( 520 )),

/*
 * vertical blank is in progress
 */
 DDERR_VERTICALBLANKINPROGRESS		=(0x88760000|( 537 )),

/*
 * Informs DirectDraw that the previous Blt which is transfering information
 * to or from this Surface is incomplete.
 */
 DDERR_WASSTILLDRAWING			=(0x88760000|( 540 )),


/*
 * Rectangle provided was not horizontally aligned on reqd. boundary
 */
 DDERR_XALIGN				=(0x88760000|( 560 )),

/*
 * The GUID passed to DirectDrawCreate is not a valid DirectDraw driver
 * identifier.
 */
 DDERR_INVALIDDIRECTDRAWGUID		=(0x88760000|( 561 )),

/*
 * A DirectDraw object representing this driver has already been created
 * for this process.
 */
 DDERR_DIRECTDRAWALREADYCREATED		=(0x88760000|( 562 )),

/*
 * A hardware only DirectDraw object creation was attempted but the driver
 * did not support any hardware.
 */
 DDERR_NODIRECTDRAWHW			=(0x88760000|( 563 )),

/*
 * this process already has created a primary surface
 */
 DDERR_PRIMARYSURFACEALREADYEXISTS	=(0x88760000|( 564 )),

/*
 * software emulation not available.
 */
 DDERR_NOEMULATION			=(0x88760000|( 565 )),

/*
 * region passed to Clipper::GetClipList is too small.
 */
 DDERR_REGIONTOOSMALL			=(0x88760000|( 566 )),

/*
 * an attempt was made to set a clip list for a clipper objec that
 * is already monitoring an hwnd.
 */
 DDERR_CLIPPERISUSINGHWND		=(0x88760000|( 567 )),

/*
 * No clipper object attached to surface object
 */
 DDERR_NOCLIPPERATTACHED			=(0x88760000|( 568 )),

/*
 * Clipper notification requires an HWND or
 * no HWND has previously been set as the CooperativeLevel HWND.
 */
 DDERR_NOHWND				=(0x88760000|( 569 )),

/*
 * HWND used by DirectDraw CooperativeLevel has been subclassed,
 * this prevents DirectDraw from restoring state.
 */
 DDERR_HWNDSUBCLASSED			=(0x88760000|( 570 )),

/*
 * The CooperativeLevel HWND has already been set.
 * It can not be reset while the process has surfaces or palettes created.
 */
 DDERR_HWNDALREADYSET			=(0x88760000|( 571 )),

/*
 * No palette object attached to this surface.
 */
 DDERR_NOPALETTEATTACHED			=(0x88760000|( 572 )),

/*
 * No hardware support for 16 or 256 color palettes.
 */
 DDERR_NOPALETTEHW			=(0x88760000|( 573 )),

/*
 * If a clipper object is attached to the source surface passed into a
 * BltFast call.
 */
 DDERR_BLTFASTCANTCLIP			=(0x88760000|( 574 )),

/*
 * No blter.
 */
 DDERR_NOBLTHW				=(0x88760000|( 575 )),

/*
 * No DirectDraw ROP hardware.
 */
 DDERR_NODDROPSHW			=(0x88760000|( 576 )),

/*
 * returned when GetOverlayPosition is called on a hidden overlay
 */
 DDERR_OVERLAYNOTVISIBLE			=(0x88760000|( 577 )),

/*
 * returned when GetOverlayPosition is called on a overlay that UpdateOverlay
 * has never been called on to establish a destionation.
 */
 DDERR_NOOVERLAYDEST			=(0x88760000|( 578 )),

/*
 * returned when the position of the overlay on the destionation is no longer
 * legal for that destionation.
 */
 DDERR_INVALIDPOSITION			=(0x88760000|( 579 )),

/*
 * returned when an overlay member is called for a non-overlay surface
 */
 DDERR_NOTAOVERLAYSURFACE		=(0x88760000|( 580 )),

/*
 * An attempt was made to set the cooperative level when it was already
 * set to exclusive.
 */
 DDERR_EXCLUSIVEMODEALREADYSET		=(0x88760000|( 581 )),

/*
 * An attempt has been made to flip a surface that is not flippable.
 */
 DDERR_NOTFLIPPABLE			=(0x88760000|( 582 )),

/*
 * Can't duplicate primary & 3D surfaces, or surfaces that are implicitly
 * created.
 */
 DDERR_CANTDUPLICATE			=(0x88760000|( 583 )),

/*
 * Surface was not locked.  An attempt to unlock a surface that was not
 * locked at all, or by this process, has been attempted.
 */
 DDERR_NOTLOCKED				=(0x88760000|( 584 )),

/*
 * Windows can not create any more DCs
 */
 DDERR_CANTCREATEDC			=(0x88760000|( 585 )),

/*
 * No DC was ever created for this surface.
 */
 DDERR_NODC				=(0x88760000|( 586 )),

/*
 * This surface can not be restored because it was created in a different
 * mode.
 */
 DDERR_WRONGMODE				=(0x88760000|( 587 )),

/*
 * This surface can not be restored because it is an implicitly created
 * surface.
 */
 DDERR_IMPLICITLYCREATED			=(0x88760000|( 588 )),

/*
 * The surface being used is not a palette-based surface
 */
 DDERR_NOTPALETTIZED			=(0x88760000|( 589 )),


/*
 * The display is currently in an unsupported mode
 */
 DDERR_UNSUPPORTEDMODE			=(0x88760000|( 590 )),

/*
 * Operation could not be carried out because there is no mip-map
 * texture mapping hardware present or available.
 */
 DDERR_NOMIPMAPHW			=(0x88760000|( 591 )),

/*
 * The requested action could not be performed because the surface was of
 * the wrong type.
 */
 DDERR_INVALIDSURFACETYPE                =(0x88760000|( 592 )),



/*
 * Device does not support optimized surfaces, therefore no video memory optimized surfaces
 */
 DDERR_NOOPTIMIZEHW                      =(0x88760000|( 600 )),

/*
 * Surface is an optimized surface, but has not yet been allocated any memory
 */
 DDERR_NOTLOADED                         =(0x88760000|( 601 )),

/*
 * Attempt was made to create or set a device window without first setting
 * the focus window
 */
 DDERR_NOFOCUSWINDOW                     =(0x88760000|( 602 )),

/*
 * A DC has already been returned for this surface. Only one DC can be
 * retrieved per surface.
 */
 DDERR_DCALREADYCREATED			=(0x88760000|( 620 )),

/*
 * An attempt was made to allocate non-local video memory from a device
 * that does not support non-local video memory.
 */
 DDERR_NONONLOCALVIDMEM                  =(0x88760000|( 630 )),

/*
 * The attempt to page lock a surface failed.
 */
 DDERR_CANTPAGELOCK			=(0x88760000|( 640 )),

/*
 * The attempt to page unlock a surface failed.
 */
 DDERR_CANTPAGEUNLOCK			=(0x88760000|( 660 )),

/*
 * An attempt was made to page unlock a surface with no outstanding page locks.
 */
 DDERR_NOTPAGELOCKED			=(0x88760000|( 680 )),

/*
 * There is more data available than the specified buffer size could hold
 */
 DDERR_MOREDATA         			=(0x88760000|( 690 )),

/*
 * The video port is not active
 */
 DDERR_VIDEONOTACTIVE   			=(0x88760000|( 695 )),

/*
 * Surfaces created by one direct draw device cannot be used directly by
 * another direct draw device.
 */
 DDERR_DEVICEDOESNTOWNSURFACE   		=(0x88760000|( 699 )),


/*
 * An attempt was made to invoke an interface member of a DirectDraw object
 * created by CoCreateInstance()), before it was initialized.
 */
// DDERR_NOTINITIALIZED                    =cast(uint)CO_E_NOTINITIALIZED
 DDERR_NOTINITIALIZED                    =0x800401F0
}

enum : uint {
/****************************************************************************
 *
 * DIRECTDRAWSURFACE LOCK FLAGS
 *
 ****************************************************************************/

/*
 * The default.	 Set to indicate that Lock should return a valid memory pointer
 * to the top of the specified rectangle.  If no rectangle is specified then a
 * pointer to the top of the surface is returned.
 */
DDLOCK_SURFACEMEMORYPTR			= 0x00000000,	// default

/*
 * Set to indicate that Lock should wait until it can obtain a valid memory
 * pointer before returning.  If this bit is set, Lock will never return
 * DDERR_WASSTILLDRAWING.
 */
DDLOCK_WAIT				 = 0x00000001,

/*
 * Set if an event handle is being passed to Lock.  Lock will trigger the event
 * when it can return the surface memory pointer requested.
 */
DDLOCK_EVENT				 = 0x00000002,

/*
 * Indicates that the surface being locked will only be read from.
 */
DDLOCK_READONLY				 = 0x00000010,

/*
 * Indicates that the surface being locked will only be written to
 */
DDLOCK_WRITEONLY			= 0x00000020,


/*
 * Indicates that a system wide lock should not be taken when this surface
 * is locked. This has several advantages (cursor responsiveness, ability
 * to call more Windows functions, easier debugging) when locking video
 * memory surfaces. However, an application specifying this flag must
 * comply with a number of conditions documented in the help file.
 * Furthermore, this flag cannot be specified when locking the primary.
 */
DDLOCK_NOSYSLOCK                        = 0x00000800
}

enum : uint {
/****************************************************************************
 *
 * DIRECTDRAW BLT FLAGS
 *
 ****************************************************************************/

/*
 * Use the alpha information in the pixel format or the alpha channel surface
 * attached to the destination surface as the alpha channel for this blt.
 */
DDBLT_ALPHADEST				 = 0x00000001,

/*
 * Use the dwConstAlphaDest field in the DDBLTFX structure as the alpha channel
 * for the destination surface for this blt.
 */
DDBLT_ALPHADESTCONSTOVERRIDE		 = 0x00000002,

/*
 * The NEG suffix indicates that the destination surface becomes more
 * transparent as the alpha value increases. (0 is opaque)
 */
DDBLT_ALPHADESTNEG			= 0x00000004,

/*
 * Use the lpDDSAlphaDest field in the DDBLTFX structure as the alpha
 * channel for the destination for this blt.
 */
DDBLT_ALPHADESTSURFACEOVERRIDE		= 0x00000008,

/*
 * Use the dwAlphaEdgeBlend field in the DDBLTFX structure as the alpha channel
 * for the edges of the image that border the color key colors.
 */
DDBLT_ALPHAEDGEBLEND			 = 0x00000010,

/*
 * Use the alpha information in the pixel format or the alpha channel surface
 * attached to the source surface as the alpha channel for this blt.
 */
DDBLT_ALPHASRC				 = 0x00000020,

/*
 * Use the dwConstAlphaSrc field in the DDBLTFX structure as the alpha channel
 * for the source for this blt.
 */
DDBLT_ALPHASRCCONSTOVERRIDE		 = 0x00000040,

/*
 * The NEG suffix indicates that the source surface becomes more transparent
 * as the alpha value increases. (0 is opaque)
 */
DDBLT_ALPHASRCNEG			 = 0x00000080,

/*
 * Use the lpDDSAlphaSrc field in the DDBLTFX structure as the alpha channel
 * for the source for this blt.
 */
DDBLT_ALPHASRCSURFACEOVERRIDE		 = 0x00000100,

/*
 * Do this blt asynchronously through the FIFO in the order received.  If
 * there is no room in the hardware FIFO fail the call.
 */
DDBLT_ASYNC				 = 0x00000200,

/*
 * Uses the dwFillColor field in the DDBLTFX structure as the RGB color
 * to fill the destination rectangle on the destination surface with.
 */
DDBLT_COLORFILL				 = 0x00000400,

/*
 * Uses the dwDDFX field in the DDBLTFX structure to specify the effects
 * to use for the blt.
 */
DDBLT_DDFX				 = 0x00000800,

/*
 * Uses the dwDDROPS field in the DDBLTFX structure to specify the ROPS
 * that are not part of the Win32 API.
 */
DDBLT_DDROPS				 = 0x00001000,

/*
 * Use the color key associated with the destination surface.
 */
DDBLT_KEYDEST				 = 0x00002000,

/*
 * Use the dckDestColorkey field in the DDBLTFX structure as the color key
 * for the destination surface.
 */
DDBLT_KEYDESTOVERRIDE			= 0x00004000,

/*
 * Use the color key associated with the source surface.
 */
DDBLT_KEYSRC				=0x00008000,

/*
 * Use the dckSrcColorkey field in the DDBLTFX structure as the color key
 * for the source surface.
 */
DDBLT_KEYSRCOVERRIDE			=0x00010000,

/*
 * Use the dwROP field in the DDBLTFX structure for the raster operation
 * for this blt.  These ROPs are the same as the ones defined in the Win32 API.
 */
DDBLT_ROP				=0x00020000,

/*
 * Use the dwRotationAngle field in the DDBLTFX structure as the angle
 * (specified in 1/100th of a degree) to rotate the surface.
 */
DDBLT_ROTATIONANGLE			=0x00040000,

/*
 * Z-buffered blt using the z-buffers attached to the source and destination
 * surfaces and the dwZBufferOpCode field in the DDBLTFX structure as the
 * z-buffer opcode.
 */
DDBLT_ZBUFFER				=0x00080000,

/*
 * Z-buffered blt using the dwConstDest Zfield and the dwZBufferOpCode field
 * in the DDBLTFX structure as the z-buffer and z-buffer opcode respectively
 * for the destination.
 */
DDBLT_ZBUFFERDESTCONSTOVERRIDE		= 0x00100000,

/*
 * Z-buffered blt using the lpDDSDestZBuffer field and the dwZBufferOpCode
 * field in the DDBLTFX structure as the z-buffer and z-buffer opcode
 * respectively for the destination.
 */
DDBLT_ZBUFFERDESTOVERRIDE		=0x00200000,

/*
 * Z-buffered blt using the dwConstSrcZ field and the dwZBufferOpCode field
 * in the DDBLTFX structure as the z-buffer and z-buffer opcode respectively
 * for the source.
 */
DDBLT_ZBUFFERSRCCONSTOVERRIDE		=0x00400000,

/*
 * Z-buffered blt using the lpDDSSrcZBuffer field and the dwZBufferOpCode
 * field in the DDBLTFX structure as the z-buffer and z-buffer opcode
 * respectively for the source.
 */
DDBLT_ZBUFFERSRCOVERRIDE                =0x00800000,

/*
 * wait until the device is ready to handle the blt
 * this will cause blt to not return DDERR_WASSTILLDRAWING
 */
DDBLT_WAIT                              =0x01000000,

/*
 * Uses the dwFillDepth field in the DDBLTFX structure as the depth value
 * to fill the destination rectangle on the destination Z-buffer surface
 * with.
 */
DDBLT_DEPTHFILL				=0x02000000,



/****************************************************************************
 *
 * BLTFAST FLAGS
 *
 ****************************************************************************/

DDBLTFAST_NOCOLORKEY                    =0x00000000,
DDBLTFAST_SRCCOLORKEY                   =0x00000001,
DDBLTFAST_DESTCOLORKEY                  =0x00000002,
DDBLTFAST_WAIT                          =0x00000010,

/****************************************************************************
 *
 * FLIP FLAGS
 *
 ****************************************************************************/

DDFLIP_WAIT                          =0x00000001,

/*
 * Indicates that the target surface contains the even field of video data.
 * This flag is only valid with an overlay surface.
 */
DDFLIP_EVEN                          =0x00000002,

/*
 * Indicates that the target surface contains the odd field of video data.
 * This flag is only valid with an overlay surface.
 */
DDFLIP_ODD                           =0x00000004

}


enum : uint {
/****************************************************************************
 *
 * DIRECTDRAW WAITFORVERTICALBLANK FLAGS
 *
 ****************************************************************************/

/*
 * return when the vertical blank interval begins
 */
DDWAITVB_BLOCKBEGIN			 = 0x00000001,

/*
 * set up an event to trigger when the vertical blank begins
 */
DDWAITVB_BLOCKBEGINEVENT		= 0x00000002,

/*
 * return when the vertical blank interval ends and display begins
 */
DDWAITVB_BLOCKEND			 = 0x00000004

}
